-- PostgreSql
-- ==========

DROP TABLE IF EXISTS books;
DROP TABLE IF EXISTS publishers;
DROP TABLE IF EXISTS locations;

CREATE TABLE locations (
    location_id INT PRIMARY KEY,
    city        VARCHAR(100) NOT NULL
);

CREATE TABLE publishers (
    publisher_id INT PRIMARY KEY,
    name         VARCHAR(100) NOT NULL,
    location_id  INT NOT NULL,
    CONSTRAINT   fk_location FOREIGN KEY(location_id) REFERENCES locations(location_id)
);

CREATE TABLE books (
    book_id      INT PRIMARY KEY,
    title        VARCHAR(100) NOT NULL,
    author       VARCHAR(100) NOT NULL,
    publisher_id INT NOT NULL,
    CONSTRAINT   fk_publisher FOREIGN KEY(publisher_id) REFERENCES publishers(publisher_id)
);

INSERT INTO locations (location_id, city) VALUES (51, 'London');
INSERT INTO locations (location_id, city) VALUES (52, 'New York');
INSERT INTO locations (location_id, city) VALUES (53, 'New Delhi');
INSERT INTO locations (location_id, city) VALUES (54, 'Tokyo');
INSERT INTO locations (location_id, city) VALUES (55, 'Manila');

INSERT INTO publishers (publisher_id, name, location_id) VALUES (71, 'Bare Thoughts Publishers', 51);
INSERT INTO publishers (publisher_id, name, location_id) VALUES (72, 'TINONAY Books', 52);
INSERT INTO publishers (publisher_id, name, location_id) VALUES (73, 'God''s Bubble Inc', 53);
INSERT INTO publishers (publisher_id, name, location_id) VALUES (74, 'Dil Mil Publishing', 52);

INSERT INTO books (book_id, title, author, publisher_id) VALUES (91, 'The Captive', 'Lavanya Nukavarapu', 72);
INSERT INTO books (book_id, title, author, publisher_id) VALUES (92, 'Transition & other stories', 'Kalam Babu', 71);

DROP TABLE IF EXISTS employees;
DROP TABLE IF EXISTS departments;

CREATE TABLE departments (
	department_id INT PRIMARY KEY,
	name          VARCHAR(100) NOT NULL
);

CREATE TABLE employees (
	employee_id   INT PRIMARY KEY,
	first_name    VARCHAR(100) NOT NULL,
	last_name     VARCHAR(100) NOT NULL,
	manager_id    INT DEFAULT NULL,
	department_id INT NOT NULL,
    FOREIGN KEY  (manager_id)    REFERENCES employees (employee_id),
	FOREIGN KEY  (department_id) REFERENCES departments (department_id)
);

INSERT INTO departments (department_id, name) VALUES (1000, 'Administration');
INSERT INTO departments (department_id, name) VALUES (2000, 'Marketing');
INSERT INTO departments (department_id, name) VALUES (3000, 'Purchasing');
INSERT INTO departments (department_id, name) VALUES (4000, 'Human Resources');
INSERT INTO departments (department_id, name) VALUES (5000, 'IT');

INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (1001, 'Ajay',     'Mathur',     NULL, 1000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (1002, 'Sumitra',  'Veni',       1001, 1000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (1003, 'Babu',     'Rao',        1001, 1000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (2001, 'John',     'Denver',     NULL, 2000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (2002, 'Venkat',   'Reddy',      2001, 2000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (2003, 'Susanne',  'Collins',    2001, 2000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (2004, 'Sushma',   'Khanna',     2001, 2000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (3001, 'Krishnam', 'Raju',       NULL, 3000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (3002, 'Gautam',   'Venkatesan', 3001, 3000 );
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (3003, 'Kritika',  'Shah',       3001, 3000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (3004, 'Sai',      'Janardhan',  3001, 3000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (3005, 'Surya',    'Gowda',      3001, 3000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (4001, 'Nataly',   'Dashnov',    NULL, 4000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (4002, 'Mary',     'Underwood',  4001, 4000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (4003, 'Savita',   'Murthy',     4001, 4000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (5001, 'Dmitry',   'Ivanov',     NULL, 4000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (5002, 'Arvind',   'Shetty',     5001, 5000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (5003, 'Mihir',    'Deshpande',  5001, 5000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (5004, 'Jagadeka', 'Veerudu',    5001, 5000);
INSERT INTO employees (employee_id, first_name, last_name, manager_id, department_id) VALUES (5005, 'Atiloka',  'Sundari',    5001, 5000);

-- INNER JOIN
-- ==========
SELECT *
FROM books b
INNER JOIN publishers p
ON (b.publisher_id = p.publisher_id);

SELECT b.title, b.author, p.name publisher
FROM books b
JOIN publishers p
ON (b.publisher_id = p.publisher_id);


-- LEFT OUTER JOIN
-- ===============
SELECT *
FROM locations l
LEFT OUTER JOIN publishers p
ON (l.location_id = p.location_id);

SELECT l.*, p.name publisher_name
FROM locations l
LEFT JOIN publishers p
ON (l.location_id = p.location_id)
ORDER BY l.location_id;

SELECT l.*, p.name publisher_name, b.title, b.author
FROM locations l
LEFT JOIN publishers p
ON (l.location_id = p.location_id)
LEFT JOIN books b
ON (p.publisher_id = b.publisher_id)
ORDER BY l.location_id, p.publisher_id, b.book_id;

-- RIGHT OUTER JOIN
-- ================
SELECT *
FROM books b
RIGHT OUTER JOIN publishers p
ON (p.publisher_id = b.publisher_id);

SELECT b.title, b.author, p.name publisher_name
FROM books b
RIGHT JOIN publishers p
ON (p.publisher_id = b.publisher_id);

SELECT b.title, b.author, p.name publisher_name, l.city
FROM books b
RIGHT JOIN publishers p
ON (p.publisher_id = b.publisher_id)
RIGHT JOIN locations l
ON (l.location_id = p.location_id)
ORDER BY l.location_id, p.publisher_id, b.book_id;

-- FULL OUTER JOIN
-- ===============
SELECT *
FROM locations l
FULL OUTER JOIN publishers p
ON (l.location_id = p.location_id);

SELECT l.city, p.name publisher_name, b.title, b.author
FROM locations l
FULL JOIN publishers p
ON (l.location_id = p.location_id)
FULL JOIN books b
ON (p.publisher_id = b.publisher_id)
ORDER BY l.city;

-- SELF JOIN
-- =========
SELECT e1.first_name, e1.last_name, e2.first_name mgr_first_name, e2.last_name mgr_last_name
FROM employees e1, employees e2
WHERE e1.manager_id = e2.employee_id;

SELECT e1.first_name, e1.last_name, e2.first_name mgr_first_name, e2.last_name mgr_last_name
FROM employees e1
JOIN employees e2
ON e1.manager_id = e2.employee_id;

-- SEMI JOIN
-- =========
SELECT e.first_name, e.last_name, d.name department_name
FROM employees e, departments d
WHERE e.department_id = d.department_id
AND d.department_id IN
(SELECT d.department_id
WHERE d.department_id BETWEEN 2000 AND 4000
);

SELECT e.first_name, e.last_name, d.name department_name
FROM employees e, departments d
WHERE e.department_id = d.department_id
AND EXISTS 
(SELECT d.department_id
WHERE d.department_id BETWEEN 2000 AND 4000
);

-- ANTI JOIN
-- =========
SELECT e.first_name, e.last_name, d.name department_name
FROM employees e, departments d
WHERE e.department_id = d.department_id
AND NOT EXISTS 
(SELECT d.department_id
WHERE d.department_id BETWEEN 2000 AND 4000
);